@@ -13,11 +13,12 @@ import sys
 import termcolor
 import termios
 import time
+import logging
 
 from twisted.internet import fdesc
 from deployer.pty import select
 from deployer.std import raw_mode
-from deployer.console import in_columns, lesspipe
+from deployer.console import Console
 
 
 def commonprefix(*strings):
@@ -107,6 +108,8 @@ class CLInterface(object):
         h = self.root
         parts = parts[:]
 
+        logging.info('Handle command line action "%s"' % ' '.join(parts))
+
         while h and parts:# and not h.is_leaf:
             try:
                 h = h.get_subhandler(parts[0])
@@ -165,7 +168,8 @@ class CLInterface(object):
                 length = len(w[0]) + len(handler_type.postfix)
                 yield text, length
 
-        lesspipe(in_columns(column_items(), self.pty), self.pty)
+        c = Console(self.pty)
+        c.lesspipe(c.in_columns(column_items()))
 
     def ctrl_c(self):
         # Initialize new read
@@ -578,7 +582,7 @@ class CLInterface(object):
                     else:
                         c = self.stdin.read(1)
 
-                    if c == '[': # (91)
+                    if c in ('[', 'O'): # (91, 68)
                         c = self.stdin.read(1)
 
                         # Cursor to left
@@ -637,17 +641,6 @@ class CLInterface(object):
                         elif c == 'F':
                             self.end()
 
-                    elif c == 'O':
-                        c = self.stdin.read(1)
-
-                        # Home
-                        if c == 'H':
-                            self.home()
-
-                        # End
-                        elif c == 'F':
-                            self.end()
-
                 # Insert character
                 else:
                     if self.vi_navigation:
