@@ -1,4 +1,3 @@
-from deployer.console import confirm
 from deployer.contrib.commands import wget
 from deployer.contrib.services.apt_get import AptGet
 from deployer.contrib.services.config import Config
@@ -6,6 +5,7 @@ from deployer.contrib.services.upstart import UpstartService
 from deployer.query import Q
 from deployer.service import Service, required_property, isolate_host
 from pygments.lexers import IniLexer
+from deployer.utils import esc1
 
 
 # /etc/redis.conf template
@@ -30,6 +30,9 @@ port %(port)s
 # Close the connection after a client is idle for N seconds (0 to disable)
 timeout %(timeout)s
 
+# Logfile
+logfile %(logfile)s
+
 %(bind)s
 """
 
@@ -54,7 +57,7 @@ class Redis(Service):
     bind = None
 
     # Download URL
-    redis_download_url = 'http://redis.googlecode.com/files/redis-2.6.11.tar.gz'
+    redis_download_url = 'http://redis.googlecode.com/files/redis-2.6.13.tar.gz'
 
     # directory for the database file, or None for the home directory
     @property
@@ -73,6 +76,10 @@ class Redis(Service):
     def config_file(self):
         return '/etc/redis-%s.conf' % self.slug
 
+    @property
+    def logfile(self):
+        return '/var/log/redis-%s.log' % self.slug
+
     class packages(AptGet):
         # Packages required for building redis
         packages = ('make', 'gcc',  'telnet', 'build-essential')
@@ -94,7 +101,7 @@ class Redis(Service):
         # Also make sure that redis was not yet installed
         if self.is_already_installed:
             print 'Warning: Redis is already installed'
-            if not confirm('Redis is already installed. Reinstall?'):
+            if not self.console.confirm('Redis is already installed. Reinstall?'):
                 return
 
         # Install dependencies
@@ -119,6 +126,39 @@ class Redis(Service):
                     self.host.sudo('make install')
 
         self.config.setup()
+        self.upstart_service.setup()
+        self.touch_logfile()
+
+    def tail_logfile(self):
+        self.host.sudo("tail -n 20 -f '%s'" % esc1(self.logfile))
+
+    @property
+    def is_already_installed(self):
+        """
+        Returns true when redis was already installed on all hosts
+        """
+        return self.host.exists(self.config_file) and self.upstart_service.is_already_installed()
+
+
+    def shell(self):
+        print 'Opening telnet connection to Redis... Press Ctrl-C to exit.'
+        print
+        self.host.run('redis-cli -h localhost -a "%s" -p %s' % (self.password or '', self.port))
+
+
+    def monitor(self):
+        """
+        Monitor all commands that are currently executed on this redis database.
+        """
+        self.host.run('echo "MONITOR" | redis-cli -h localhost -a "%s" -p %s' % (self.password or '', self.port))
+
+
+    def dbsize(self):
+        """
+        Return the number of keys in the selected database.
+        """
+
+
 
         # Install upstart config, and run
         self.upstart_service.setup()
@@ -142,12 +182,20 @@ class Redis(Service):
                     'auto_save': 'save 60 1' if self.persistent else '',
                     'bind': ('bind %s' %  self.bind if self.bind else ''),
                     'timeout': str(int(self.timeout)),
+                    'logfile': self.logfile,
                 }
 
         def setup(self):
             Config.setup(self)
             self.host.sudo("chown '%s' '%s' " % (self.parent.username, self.remote_path))
 
+    def touch_logfile(self):
+        # Touch and chown logfile.
+        self.host.sudo("touch '%s'" % esc1(self.logfile))
+        self.host.sudo("chown '%s' '%s'" % (esc1(self.username), esc1(self.logfile)))
+
+    def tail_logfile(self):
+        self.host.sudo("tail -n 20 -f '%s'" % esc1(self.logfile))
 
     @property
     def is_already_installed(self):
