@@ -10,6 +10,7 @@
 import StringIO
 import copy
 import getpass
+import logging
 import os
 import paramiko
 import pexpect
@@ -22,10 +23,11 @@ import threading
 import time
 import tty
 
-from deployer.console import input as read_input
+from deployer.console import Console
 from deployer.exceptions import ExecCommandFailed
 from deployer.loggers import DummyLoggerInterface, Actions
 from deployer.pty import DummyPty, select
+from deployer.std import raw_mode
 from deployer.utils import esc1
 
 from twisted.internet import fdesc
@@ -56,7 +58,7 @@ class Host(object):
     start_path = None # None or string
 
     # Terminal to report to use for interactive sessions
-    term = 'xterm' # xterm, vt100, xterm-256color
+    term = os.environ.get('TERM', 'xterm') # xterm, vt100, xterm-256color
 
     # Magic prompt. We expect this string to not appear in the stdout of
     # random programs. This makes it possible to automatically send the
@@ -135,6 +137,10 @@ class Host(object):
         """
         result = []
 
+        # Ensure that the start-path exists (only if one was given. Not for the ~)
+        if self.start_path:
+            result.append("(mkdir -p '%s' 2> /dev/null || true) && " % esc1(self.expand_path(self.start_path)))
+
         # Prefix with all cd-statements
         for p in [self._get_start_path()] + self._path:
             # TODO: We can't have double quotes around paths,
@@ -212,6 +218,9 @@ class Host(object):
                 command = "bash -n -c '%s' " % esc1(command)
                 command = "%s;echo '%s'" % (command, esc1(command))
 
+            logging.info('Running "%s" on host "%s" sudo=%r, interactive=%r' %
+                            (command, self.slug, use_sudo, interactive))
+
             # Execute
             if use_sudo:
                 # We use 'sudo su' instead of 'sudo -u', because shell expension
@@ -223,21 +232,27 @@ class Host(object):
                 # 2. This shows the home directory of the user postgres:
                 # sudo su postgres -c 'echo $HOME '
                 if interactive:
-                    chan.exec_command(self._wrap_command(
+                    wrapped_command = self._wrap_command(
                                 "sudo -p '%s' su '%s' -c '%s'" % (esc1(self.magic_sudo_prompt), esc1(user), esc1(command))
                                 #"sudo -u '%s' bash -c '%s'" % (user, esc1(command))
                                 if user else
                                 "sudo -p '%s' bash -c '%s' " % (esc1(self.magic_sudo_prompt), esc1(command))
-                                ))
+                                )
+
+                    logging.debug('Running wrapped command "%s"' % wrapped_command)
+                    chan.exec_command(wrapped_command)
 
                 # Some commands, like certain /etc/init.d scripts cannot be
                 # run interactively. They won't work in a ssh pty.
                 else:
-                    chan.exec_command(self._wrap_command(
+                    wrapped_command = self._wrap_command(
                         "echo '%s' | sudo -p '(passwd)' -u '%s' -P %s " % (esc1(self.password), esc1(user), command)
                         if user else
                         "echo '%s' | sudo -p '(passwd)' -S %s " % (esc1(self.password), command)
-                        ))
+                        )
+
+                    logging.debug('Running wrapped command "%s" interactive' % wrapped_command)
+                    chan.exec_command(wrapped_command)
             else:
                 chan.exec_command(self._wrap_command(command))
 
@@ -311,88 +326,100 @@ class Host(object):
         # bytes as possible without blocking.)
         fdesc.setNonBlocking(pty.stdin)
 
-        try:
-            # Set terminal raw
-            if raw:
-                tty.setraw(pty.stdin.fileno())
-                tty.setcbreak(pty.stdin.fileno())
-            chan.settimeout(0.0)
+        import contextlib
 
-            # When initial input has been given, send this first
-            if initial_input:
-                time.sleep(0.2) # Wait a very short while for the channel to be initialized, before sending.
-                chan.send(initial_input)
+        # Set terminal in raw mode
+        if raw:
+            context = raw_mode(pty.stdin)
+        else:
+            context = contextlib.nested()
 
-            # Read/write loop
-            while True:
-                # Don't wait for any input when an exit status code has been
-                # set already.
-                if chan.status_event.isSet():
-                    break;
+        with context:
+            try:
+                chan.settimeout(0.0)
 
-                r, w, e = select([pty.stdin, chan], [], [])
+                # When initial input has been given, send this first
+                if initial_input:
+                    time.sleep(0.2) # Wait a very short while for the channel to be initialized, before sending.
+                    chan.send(initial_input)
 
-                # Receive stream
-                if chan in r:
-                    try:
-                        x = chan.recv(1024)
+                # Read/write loop
+                while True:
+                    # Don't wait for any input when an exit status code has been
+                    # set already.
+                    if chan.status_event.isSet():
+                        break;
+
+                    r, w, e = select([pty.stdin, chan], [], [])
+
+                    # Receive stream
+                    if chan in r:
+                        try:
+                            x = chan.recv(1024)
+
+                            # Received length 0 -> end of stream
+                            if len(x) == 0:
+                                break
+
+                            # Log received characters
+                            log_entry.log_io(x)
+
+                            # Write received characters to stdout and flush
+                            while True:
+                                try:
+                                    pty.stdout.write(x)
+                                    break
+                                except IOError, e:
+                                    # Sometimes, when we have a lot of output, we get here:
+                                    # IOError: [Errno 11] Resource temporarily unavailable
+                                    # Just waiting a little, and retrying seems to work.
+                                    # See also: deployer.run.socket_client for a similar issue.
+                                    time.sleep(0.2)
+
+                            pty.stdout.flush()
+
+                            # Also remember received output.
+                            # We want to return what's written to stdout.
+                            result.append(x)
+
+                            # Do we need to send the sudo password? (It's when the
+                            # magic prompt has been detected in the stream) Note
+                            # that we only monitor the last part of 'result', it's
+                            # a bit fuzzy, but works.
+                            if not password_sent and self.magic_sudo_prompt in ''.join(result[-32:]):
+                                chan.send(self.password)
+                                chan.send('\n')
+                                password_sent = True
+                        except socket.timeout:
+                            pass
+
+                    # Send stream (one by one character)
+                    if pty.stdin in r:
+                        try:
+                            x = pty.stdin.read(1024)
+                        except IOError, e:
+                            # What to do with IOError exceptions?
+                            # (we see what happens in the next select-call.)
+                            continue
 
                         # Received length 0 -> end of stream
                         if len(x) == 0:
                             break
 
-                        # Log received characters
-                        log_entry.log_io(x)
+                        # Write to channel
+                        chan.send(x)
 
-                        # Write received characters to stdout and flush
-                        pty.stdout.write(x)
-                        pty.stdout.flush()
+                        # Not sure about this. Sometimes, when pasting large data
+                        # in the command line, the many concecutive read or write
+                        # commands will make Paramiko hang somehow...  (This was
+                        # the case, when still using a blocking pty.stdin.read(1)
+                        # instead of a non-blocking readmany.
+                        time.sleep(0.01)
+            finally:
+                # Set blocking again
+                fdesc.setBlocking(pty.stdin)
 
-                        # Also remember received output.
-                        # We want to return what's written to stdout.
-                        result.append(x)
-
-                        # Do we need to send the sudo password? (It's when the
-                        # magic prompt has been detected in the stream) Note
-                        # that we only monitor the last part of 'result', it's
-                        # a bit fuzzy, but works.
-                        if not password_sent and self.magic_sudo_prompt in ''.join(result[-32:]):
-                            chan.send(self.password)
-                            chan.send('\n')
-                            password_sent = True
-                    except socket.timeout:
-                        pass
-
-                # Send stream (one by one character)
-                if pty.stdin in r:
-                    try:
-                        x = pty.stdin.read(1024)
-                    except IOError, e:
-                        # What to do with IOError exceptions?
-                        # (we see what happens in the next select-call.)
-                        continue
-
-                    # Received length 0 -> end of stream
-                    if len(x) == 0:
-                        break
-
-                    # Write to channel
-                    chan.send(x)
-
-                    # Not sure about this. Sometimes, when pasting large data
-                    # in the command line, the many concecutive read or write
-                    # commands will make Paramiko hang somehow...  (This was
-                    # the case, when still using a blocking pty.stdin.read(1)
-                    # instead of a non-blocking readmany.
-                    time.sleep(0.01)
-        finally:
-            # Restore terminal
-            termios.tcsetattr(pty.stdin, termios.TCSADRAIN, oldtty)
-
-        # Set blocking again
-        fdesc.setBlocking(pty.stdin)
-
-        return ''.join(result)
+            return ''.join(result)
 
 
     # =====[ Actions which are also available in non-sandbox mode ]====
@@ -412,6 +439,9 @@ class Host(object):
                         """ cut -d: -f2 | awk '{ print $1}' """ % interface).strip()
 
     def get_home_directory(self, username=None):
+        # TODO:....
+        # return self.expand_path('~%s' % username if username else '~')
+
         with self.cd('/'):
             with self.sandbox(False):
                 if username:
@@ -648,6 +678,7 @@ class Host(object):
         Returns True when this file exists.
         """
         try:
+            filename = self.expand_path(filename)
             self._run_silent_sudo("test -f '%s' || test -d '%s'" % (esc1(filename), esc1(filename)))
             return True
         except ExecCommandFailed:
@@ -773,42 +804,38 @@ class SSHBackend(object):
         """
         # Lock: be sure not to create this connection from several threads at
         # the same time.
-        self._lock.acquire()
+        with self._lock:
+            if not (self._ssh_cache and self._ssh_cache._transport and self._ssh_cache._transport.is_active()):
+                h = self._get_host_instance()
 
-        if not (self._ssh_cache and self._ssh_cache._transport and self._ssh_cache._transport.is_active()):
-            h = self._get_host_instance()
+                # Show connecting message (in current stdout)
+                sys.stdout.write('*** Connecting to %s (%s)...\n' % (h.address, h.slug))
+                sys.stdout.flush()
 
-            # Show connecting message (in current stdout)
-            sys.stdout.write('*** Connecting to %s (%s)...\n' % (h.address, h.slug))
-            sys.stdout.flush()
+                # Connect
+                self._ssh_cache = paramiko.SSHClient()
 
-            # Connect
-            self._ssh_cache = paramiko.SSHClient()
+                if not h.reject_unknown_hosts:
+                    self._ssh_cache.set_missing_host_key_policy(paramiko.AutoAddPolicy())
 
-            if not h.reject_unknown_hosts:
-                self._ssh_cache.set_missing_host_key_policy(paramiko.AutoAddPolicy())
+                try:
+                    # Paramiko's authentication method can be either a public key, public key file, or password.
+                    if h.rsa_key:
+                        # RSA key
+                        rsa_key_file_obj = StringIO.StringIO(h.rsa_key)
+                        kw = { "pkey": paramiko.RSAKey.from_private_key(rsa_key_file_obj, h.rsa_key_password) }
+                    elif h.key_filename:
+                        kw = { "key_filename": h.key_filename }
+                    else:
+                        kw = { "password": h.password }
 
-            try:
-                # Paramiko's authentication method can be either a public key, public key file, or password.
-                if h.rsa_key:
-                    # RSA key
-                    rsa_key_file_obj = StringIO.StringIO(h.rsa_key)
-                    kw = { "pkey": paramiko.RSAKey.from_private_key(rsa_key_file_obj, h.rsa_key_password) }
-                elif h.key_filename:
-                    kw = { "key_filename": h.key_filename }
-                else:
-                    kw = { "password": h.password }
+                    self._ssh_cache.connect(h.address, port=h.port, username=h.username, timeout=h.timeout, **kw)
 
-                self._ssh_cache.connect(h.address, port=h.port, username=h.username, timeout=h.timeout, **kw)
+                except (paramiko.SSHException, Exception) as e:
+                    self._ssh_cache = None
+                    raise Exception('Could not connect to host %s (%s)\n%s' % (h.slug, h.address, unicode(e)))
 
-            except (paramiko.SSHException, Exception) as e:
-                self._ssh_cache = None
-                raise Exception('Could not connect to host %s (%s)\n%s' % (h.slug, h.address, unicode(e)))
-
-        # Release lock
-        self._lock.release()
-
-        return self._ssh_cache
+            return self._ssh_cache
 
 
 class SSHHost(Host):
@@ -853,16 +880,18 @@ class SSHHost(Host):
         return paramiko.SFTPClient.from_transport(transport)
 
     def expand_path(self, path):
-        # Tilde expansion
-        if path.startswith('~'):
-            home = self.sftp.normalize('.')
-            path = path.replace('~', home, 1)
+        def expand_tilde(p):
+            if p.startswith('~/') or p == '~':
+                home = self.sftp.normalize('.')
+                return p.replace('~', home, 1)
+            else:
+                return p
 
-        # Expand remote path, using the cwd
-        if not os.path.isabs(path):
-            path = os.path.join(self.cwd, path)
-
-        return path
+        # Expand remote path, using the start path and cwd
+        if self.start_path:
+            return os.path.join(expand_tilde(self.start_path), expand_tilde(self.cwd), expand_tilde(path))
+        else:
+            return os.path.join(expand_tilde(self.cwd), expand_tilde(path))
 
     def start_interactive_shell(self, pty, command=None, logger=None, initial_input=None):
         """
@@ -918,10 +947,10 @@ class LocalHost(Host):
         Host.__init__(self)
         self._backend = backend or LocalHostBackend()
 
-    def _run(self, *a, **kw):
+    def _run(self, pty, *a, **kw):
         if kw.get('use_sudo', False):
-            self._ensure_password_is_known()
-        return Host._run(self, *a, **kw)
+            self._ensure_password_is_known(pty)
+        return Host._run(self, pty, *a, **kw)
 
     def clone(self):
         # Clone state of this host into a new Host instance.
@@ -930,17 +959,24 @@ class LocalHost(Host):
         return new_host
 
     def expand_path(self, path):
-        return os.path.expanduser(path)
+        return os.path.expanduser(path) # TODO: expansion like with SSHHost!!!!
 
-    def _ensure_password_is_known(self):
+    def _ensure_password_is_known(self, pty):
         # Make sure that we know the localhost password, before running sudo.
         tries = 0
         while not self._backend.password:
-            self._backend.password = read_input('[sudo] password for %s at %s' % (self.username, self.slug), True)
+            self._backend.password = Console(pty).input('[sudo] password for %s at %s' %
+                        (self.username, self.slug), is_password=True)
 
             # Check password
             try:
-                Host._run_silent_sudo(self, '/bin/true')
+                if self.exists('/bin/true'):
+                    Host._run_silent_sudo(self, '/bin/true')
+                elif self.exists('/usr/bin/true'):
+                    # On OS X, we have /usr/bin/true.
+                    Host._run_silent_sudo(self, '/usr/bin/true')
+                else:
+                    print 'Warning: could not find /bin/true or /usr/bin/true.'
             except ExecCommandFailed:
                 print 'Incorrect password'
                 self._backend.password = None
