@@ -53,29 +53,38 @@ class Query(object):
 
     @property
     def parent(self):
-        """
-        Go the to current parent of this service.
-        """
+        """ Go the to current parent of this service.  """
         return parent(self)
 
+    # Operator overloads
     def __mod__(self, other):
-        """
-        Module operator between two Q objects:
-
-        class service(Service):
-            my_property = Q('Some string with %s placeholder') % Q.parent.property_to_be_inserted
-        """
         return operator(self, other, lambda a, b: a % b, '%')
 
     def __add__(self, other):
-        """
-        Plus operator between two Q objects.
-
-        class service(Service):
-            my_property = Q(20) % Q.parent.add_this_property
-        """
         return operator(self, other, lambda a, b: a+b, '+')
 
+    def __sub__(self, other):
+        return operator(self, other, lambda a, b: a-b, '-')
+
+    def __mul__(self, other):
+        return operator(self, other, lambda a, b: a*b, '*')
+
+    def __div__(self, other):
+        return operator(self, other, lambda a, b: a/b, '/')
+
+    # Reverse operator overloads
+    def __radd__(self, other):
+        return operator(other, self, lambda a, b: a + b, '+')
+
+    def __rsub__(self, other):
+        return operator(other, self, lambda a, b: a - b, '-')
+
+    def __rmul__(self, other):
+        return operator(other, self, lambda a, b: a*b, '*')
+
+    def __rdiv__(self, other):
+        return operator(other, self, lambda a, b: a/b, '/')
+
     def __repr__(self):
         return '<Query: %s>' % self.__str__()
 
@@ -83,16 +92,17 @@ class Query(object):
         return 'Q'
 
     # You cannot override and, or and not:
+    # Instead, we override the bitwise operators
     # http://stackoverflow.com/questions/471546/any-way-to-override-the-and-operator-in-python
 
-    #def __and__(self, other):
-    #    return operator(self, other, lambda a, b: a and b)
+    def __and__(self, other):
+        return operator(self, other, lambda a, b: a and b)
 
-    #def __or__(self, other):
-    #    return operator(self, other, lambda a, b: a or b)
+    def __or__(self, other):
+        return operator(self, other, lambda a, b: a or b)
 
-    #def __not__(self):
-    #    return operator(self, None, lambda a, b: not a)
+    def __invert__(self):
+        return invert(self)
 
 def _resolve(query_object, instance):
     """
@@ -113,6 +123,21 @@ def _resolve(query_object, instance):
     else:
         return query_object
 
+class invert(Query):
+    """
+    Implementation of the invert operator
+    """
+    def __init__(self, part):
+        self.part = part
+
+    @property
+    def _query(self):
+        def result(instance):
+            return not instance
+        return result
+
+    def __str__(self):
+        return u'~ %s' % str(self.part)
 
 class operator(Query):
     """
