@@ -16,6 +16,7 @@ from twisted.internet.error import CannotListenError
 import StringIO
 import datetime
 import getpass
+import logging
 import os
 import pickle
 import sys
@@ -97,8 +98,8 @@ class SocketPty(Pty):
     """
     The Pty object that we pass to every shell.
     """
-    def __init__(self, stdin, stdout, run_in_new_pty):
-        Pty.__init__(self, stdin, stdout)
+    def __init__(self, stdin, stdout, run_in_new_pty, interactive):
+        Pty.__init__(self, stdin, stdout, interactive)
         self._run_in_new_ptys = run_in_new_pty
 
     def run_in_auxiliary_ptys(self, callbacks):
@@ -140,8 +141,8 @@ class Connection(object):
     for either an interactive session, for logging,
     or for a second parallel deployment.
     """
-    def __init__(self, settings, transportHandle, doneCallback):
-        self.settings = settings
+    def __init__(self, root_service, transportHandle, doneCallback, interactive):
+        self.root_service = root_service
         self.transportHandle = transportHandle
         self.doneCallback = doneCallback
         self.connection_shell = None
@@ -158,7 +159,7 @@ class Connection(object):
         stdout = os.fdopen(self.slave, 'w', 0)
 
         # Create pty object, for passing to deployment enviroment.
-        self.pty = SocketPty(stdin, stdout, self.runInNewPtys)
+        self.pty = SocketPty(stdin, stdout, self.runInNewPtys, interactive=interactive)
 
         # Start read loop
         self._startReading()
@@ -367,7 +368,7 @@ class ConnectionShell(object):
         self.logger_interface = LoggerInterface()
 
         # Run shell
-        self.shell = SocketShell(connection.settings, connection.pty,
+        self.shell = SocketShell(connection.root_service, connection.pty,
                                 self.logger_interface, clone_shell=clone_shell)
         self.cd_path = cd_path
 
@@ -412,7 +413,7 @@ class ConnectionShell(object):
         self.shell.session = self # Assign session to shell
 
         in_shell_logger = DefaultLogger(print_group=False)
-        extra_loggers = self.connection.settings.Meta.extra_loggers
+        extra_loggers = self.connection.root_service.Meta.extra_loggers
 
                 # in_shell_logger: Displaying of events in shell
         self.logger_interface.attach(in_shell_logger)
@@ -496,16 +497,23 @@ class CliClientProtocol(Protocol):
                 self.connection.pty.set_size(*data)
 
             elif action == '_get_info':
-                # Return information to client.
+                # Return information about the current server state
+                processes = [ {
+                            'service_name': c.connection_shell.shell.state._service.__class__.__name__,
+                            'service_module': c.connection_shell.shell.state._service.__module__,
+                            'running': c.connection_shell.shell.currently_running or '(Idle)'
+                    } for c in self.factory.connectionPool if c.connection_shell and c.connection_shell.shell ]
+
                 self._handle('_info', {
                             'created': self.created.isoformat(),
-                            # TODO: also return information about running ConnectionShell
-                            # objects.
+                            'root_service_name': self.connection.root_service.__class__.__name__,
+                            'root_service_module': self.connection.root_service.__module__,
+                            'processes': processes,
                     })
                 self.transport.loseConnection()
 
             elif action == 'open-new-window':
-                print 'opening new window...'
+                logging.info('Opening new window')
                 # When the client wants to open a new shell (Ctrl-N press for
                 # instance), check whether we are in an interactive session,
                 # and if so, copy this shell.
@@ -515,7 +523,7 @@ class CliClientProtocol(Protocol):
                     self._handle('open-new-window', { 'focus': True })
 
             elif action == '_start-interaction':
-                print 'creating session'
+                logging.info('Starting session')
 
                 cd_path = data.get('cd_path', None)
 
@@ -550,7 +558,7 @@ class CliClientProtocol(Protocol):
         """
         Disconnected from client.
         """
-        print 'Connection lost'
+        logging.info('Client connection lost')
 
         # Remove current connection from the factory's connection pool.
         self.factory.connectionPool.remove(self.connection)
@@ -558,18 +566,19 @@ class CliClientProtocol(Protocol):
 
         # When no more connections are left, close the reactor.
         if len(self.factory.connectionPool) == 0 and self.factory.shutdownOnLastDisconnect:
-            print 'Stopping server.'
+            logging.info('Stopping server.')
             reactor.stop()
 
     def _handle(self, action, data):
         self.transport.write(pickle.dumps((action, data)) )
 
     def connectionMade(self):
-        self.connection = Connection(self.factory.settings, self._handle, self.transport.loseConnection)
+        self.connection = Connection(self.factory.root_service, self._handle, self.transport.loseConnection,
+                        interactive=self.factory.interactive)
         self.factory.connectionPool.add(self.connection)
 
 
-def startSocketServer(settings, shutdownOnLastDisconnect):
+def startSocketServer(root_service, shutdownOnLastDisconnect, interactive):
     """
     Bind the first available unix socket.
     Return the path.
@@ -579,7 +588,8 @@ def startSocketServer(settings, shutdownOnLastDisconnect):
     factory.connectionPool = set() # List of currently, active connections
     factory.protocol = CliClientProtocol
     factory.shutdownOnLastDisconnect = shutdownOnLastDisconnect
-    factory.settings = settings
+    factory.root_service = root_service
+    factory.interactive = interactive
 
     # Search for a socket to listen on.
     i = 0
@@ -594,7 +604,7 @@ def startSocketServer(settings, shutdownOnLastDisconnect):
 
             # When 100 times failed, cancel server
             if i == 100:
-                print '100 times failed to listen on posix socket. Please clean up old sockets.'
+                logging.warning('100 times failed to listen on posix socket. Please clean up old sockets.')
                 raise
 
     return path
@@ -602,19 +612,27 @@ def startSocketServer(settings, shutdownOnLastDisconnect):
 
 # =================[ Startup]=================
 
-def start(settings, daemonized=False, shutdown_on_last_disconnect=False, thread_pool_size=50):
+def start(root_service, daemonized=False, shutdown_on_last_disconnect=False, thread_pool_size=50, interactive=True, logfile=None):
     """
     Start web server
     If daemonized, this will start the server in the background,
     and return the socket path.
     """
-    # Create settings instance
-    settings = settings()
+    # Create service instance
+    root_service = root_service()
 
     # Start server
-    path = startSocketServer(settings, shutdownOnLastDisconnect=shutdown_on_last_disconnect)
+    path = startSocketServer(root_service, shutdownOnLastDisconnect=shutdown_on_last_disconnect, interactive=interactive)
 
     def run_server():
+        # Set logging
+        if logfile:
+            logging.basicConfig(filename=logfile, level=logging.DEBUG)
+        elif not daemonized:
+            logging.basicConfig(filename='/dev/stdout', level=logging.DEBUG)
+
+        logging.info('Socket server started at %s' % path)
+
         # Thread sensitive interface for stdout/stdin
         std.setup()
 
@@ -635,8 +653,3 @@ def start(settings, daemonized=False, shutdown_on_last_disconnect=False, thread_
             return path
     else:
         run_server()
-
-
-if __name__ == '__main__':
-    from deployer.contrib.default_config import example_settings
-    start(settings=example_settings)
