@@ -9,6 +9,7 @@ from deployer.pty import Pty
 from deployer.shell import Shell, ShellHandler, GroupHandler, BuiltinType
 
 import codecs
+import logging
 import os
 import random
 import signal
@@ -124,9 +125,8 @@ class StandaloneShell(Shell):
     The shell that we provide via telnet/http exposes some additional
     commands for session and user management and logging.
     """
-    def __init__(self, settings, pty, logger_interface, history):
-        Shell.__init__(self, settings, pty, logger_interface)
-
+    def __init__(self, root_service, pty, logger_interface, history):
+        Shell.__init__(self, root_service, pty, logger_interface)
         self.history = history
 
     @property
@@ -134,72 +134,51 @@ class StandaloneShell(Shell):
         return { 'history': History, }
 
 
-def create_logger():
-    """
-    Create a logger, depending on the environment.
-    If we run in tmux, log to another pane.
-    """
-    if 'TMUX' in os.environ:
-        # Show logging information in a separate TMUX pane.
-
-        # Create fifo pipe
-        filename = '/tmp/deployment-log-%s' % ''.join(random.sample(string.ascii_letters, 20))
-        if not os.path.exists(filename):
-            os.mkfifo(filename)
-
-        # Split window, and open pipo in the new pane
-        os.popen('tmux split-window "cat %s; rm %s"; tmux last-pane ' % (filename, filename))
-
-        # Start logger in this pane
-        #logger_stdout = codecs.open(filename, encoding=sys.stdout.encoding, mode='w+', errors='replace')
-        logger_stdout = codecs.open(filename, mode='w+', errors='replace')
-        logger_stdout.write('\n\n*** Logging output only. Type commands in the other pane.\n\n')
-        logger_stdout.flush()
-        return IndentedDefaultLogger(logger_stdout)
-    else:
-        # If no tmux is available, log to stdout.
-        return DefaultLogger(sys.__stdout__)
-
-
-def start(settings):
+def start(root_service, interactive=True, cd_path=None, logfile=None):
     """
     Start the deployment shell in standalone modus. (No parrallel execution,
     no server/client. Just one interface, and everything sequential.)
     """
+    # Enable logging
+    if logfile:
+        logging.basicConfig(filename=logfile, level=logging.DEBUG)
+
     # Make sure that stdin and stdout are unbuffered
     # The alternative is to start Python with the -u option
     sys.stdin = os.fdopen(sys.stdin.fileno(), 'r', 0)
     sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)
 
     # Create Pty object
-    pty = Pty(sys.stdin, sys.stdout)
+    pty = Pty(sys.stdin, sys.stdout, interactive=interactive)
 
     def sigwinch_handler(n, frame):
         pty.trigger_resize()
     signal.signal(signal.SIGWINCH, sigwinch_handler)
 
-    # Initialize settings
-    settings = settings()
+    # Initialize service
+    root_service = root_service()
 
     # Loggers
-    logger = create_logger()
     history_logger = HistoryLogger()
-    extra_loggers = settings.Meta.extra_loggers
+    in_shell_logger = DefaultLogger(print_group=False)
+    extra_loggers = root_service.Meta.extra_loggers
 
     logger_interface = LoggerInterface()
-    logger_interface.attach(logger)
+    logger_interface.attach(in_shell_logger)
     logger_interface.attach(history_logger)
 
     for l in extra_loggers:
         logger_interface.attach(l)
 
     # Start shell command loop
-    StandaloneShell(settings, pty, logger_interface, history_logger.history).cmdloop()
+    print 'Running single threaded shell...'
+    shell = StandaloneShell(root_service, pty, logger_interface, history_logger.history)
+    if cd_path is not None:
+        shell.cd(cd_path)
+    shell.cmdloop()
 
     for l in extra_loggers:
         logger_interface.detach(l)
 
-
-if __name__ == '__main__':
-    from deployer.contrib.default_config import example_settings
-    start(settings=example_settings)
+    logger_interface.detach(in_shell_logger)
+    logger_interface.detach(history_logger)
