@@ -1,4 +1,4 @@
-from deployer.console import input
+from deployer.console import Console
 from deployer.host import Host
 from deployer.host_container import HostsContainer, HostContainer
 from deployer.service_groups import Group
@@ -428,6 +428,15 @@ class ServiceBase(type):
         wrapped_attribute = self._wrap_attribute(name, value, self.__name__)
         type.__setattr__(self, name, wrapped_attribute)
 
+    def __instancecheck__(self, instance):
+        """
+        Override isinstance operator.
+        We consider an Env object in instance of this class as well if
+        env._service is an instance.
+        """
+        return type.__instancecheck__(self, instance) or (
+                    isinstance(instance, Env) and isinstance(instance._service, self))
+
 
 class Env(object):
     """
@@ -490,6 +499,13 @@ class Env(object):
         """
         return self._service._is_isolated
 
+    @property
+    def console(self):
+        """
+        Proxy object to the pseudoterminal.
+        """
+        return Console(self._pty)
+
     @property
     def super(self):
         """
@@ -541,7 +557,8 @@ class Env(object):
                 forkname = proxy.__getname()
 
                 # Ask before forking in sandboxed mode.
-                if self._is_sandbox and input('Do you want to fork %s in sandbox?' % forkname, answers=['y', 'n']) == 'n':
+                if self._is_sandbox and not self.console.confirm('Do you want to fork %s in sandbox?' % forkname,
+                                default=False):
                     return
 
                 # Fork logger
@@ -576,9 +593,6 @@ class Env(object):
 
         return ForkProxy(getobj, getname)
 
-    def isinstance(self, class_):
-        return isinstance(self._service, class_)
-
     def __call__(self, *a, **kw):
         self._service(*a, **kw).run(self._pty, self._logger)
 
@@ -766,12 +780,6 @@ class Action(object):
         Call the action:
         >> Service('...').action()
         """
-        # Extract _trace_action from kwargs
-        trace_action = kwargs.get('_trace_action', False)
-        kwargs = kwargs.copy()
-        if trace_action:
-            del kwargs['_trace_action']
-
         # Name of this action call, to be passed to the loggers.
         service = self._service
         func = self.full_name
@@ -784,12 +792,13 @@ class Action(object):
             # instance, a CalledAction will be returned. If it is an Env
             # instance, we suppose calling from within another 'env',
             # and will call it, using the same environment settings.
-            if isinstance(args[0], Service):
-                service = args[0]
-            else:
+            if isinstance(args[0], Env):
                 # args[0] should be a Env instance
                 from_env = args[0]
                 service = args[0]._service
+            else:
+                service = args[0]
+
             args = args[1:]
 
         class CalledAction(object):
@@ -829,8 +838,7 @@ class Action(object):
                     # Multiple cells, but need to be run on only one.
                     elif len(isolations) > 1 and getattr(self._func, 'isolate_one_only', False):
                         # Ask the end-user which one to use.
-                        from deployer.console import select_service_isolation
-                        isolation_service = select_service_isolation(service)
+                        isolation_service = Console(pty).select_service_isolation(service)
 
                         # This this action on the new service.
                         return [ a._run_on_service(pty, logger_interface, sandboxed, isolation_service) ]
@@ -896,10 +904,7 @@ class Action(object):
                                 if self._is_property:
                                     return result
 
-                                elif trace_action:
-                                    return ActionResult(result, capt.value, trace.first_trace) # TODO: include isolation name.
-                                else:
-                                    return result
+                                return result
 
                             # When something goes wrong, wrap exception in
                             # 'ActionException', this allows inspection of the
@@ -932,7 +937,7 @@ class Action(object):
                     print 'An exception was raised during *sandboxed* execution.'
                     print 'If you go on, the simulation will possibly be different from real execution.'
 
-                    if input('Go on?', answers=['y', 'n']) == 'n':
+                    if not Console(pty).confirm('Go on?', default=False):
                         raise e
 
         # When an action is called unbound, but with the first parameter
@@ -987,7 +992,7 @@ def get_isolations(service):
         else:
             hosts = service.hosts
 
-        def create_isolation(name, hosts_dict):
+        def create_isolation(name, hosts_dict, made_array_cell=False):
             """
             Isolation creator helper. Creates instances of Service classes,
             passing a new hosts dictionary.
@@ -1007,7 +1012,8 @@ def get_isolations(service):
                                     name=service_name,
                                     path=service_path,
                                     parent=parent_isolation.service,
-                                    creator_service=service, is_isolated=True))
+                                    creator_service=service, is_isolated=True,
+                                    made_array_cell=made_array_cell))
 
         isolate_role = service.Meta.isolate_role
 
@@ -1017,7 +1023,7 @@ def get_isolations(service):
                 hosts_in_isolation_role = len(hosts.filter(isolate_role))
 
                 if hosts_in_isolation_role == 1:
-                    yield create_isolation(parent_name, hosts)
+                    yield create_isolation(parent_name, hosts, made_array_cell=True)
                 else:
                     # When @map_roles.just_one fails, we don't yield this
                     # service, and so disallow access due to invalid mapping.
@@ -1029,35 +1035,12 @@ def get_isolations(service):
                 # Yield all isolations of this service.
                 if len(hosts.filter(isolate_role)):
                     for h, hosts2 in hosts.iterate_isolations(isolate_role):
-                        yield create_isolation(parent_name + [h.slug], hosts2)
+                        yield create_isolation(parent_name + [h.slug], hosts2, made_array_cell=True)
         else:
             # Just link to new parent.
             yield create_isolation(parent_name, hosts)
 
 
-class ActionResult(object):
-    """
-    When an action within a service is called and the _trace_action parameter
-    is given, it will return an ActionResult, This is a wrapper around the
-    real return value, which contains also the captured stdout.
-    """
-    def __init__(self, result, output, trace):
-        # Return value
-        self.result = result
-
-        # Output written to stdout
-        self.output = output
-
-        # Trace of executed actions within this block
-        self.trace = trace
-
-    def __unicode__(self):
-        return unicode(self.result)
-
-    def __str__(self):
-        return str(self.result)
-
-
 class ActionException(Exception):
     """
     When an action fails.
@@ -1116,7 +1099,7 @@ class Service(object):
     # class Hosts(object):
     #     pass
 
-    def __init__(self, hosts=None, name=None, path=None, parent=None, creator_service=None, is_isolated=False):
+    def __init__(self, hosts=None, name=None, path=None, parent=None, creator_service=None, is_isolated=False, made_array_cell=False):
         """
         'hosts` should always be a HostContainer object.
         'name` is the string of the property, how we got to this service.
@@ -1124,13 +1107,14 @@ class Service(object):
                     (Note that equels not necessary [.parent/.parent.parent/etc... ].)
         'parent` can be a Service instance.
         'creator` can be a Service instance.
+        'made_array_cell` True when moved from a not-isolated ArrayService, to the isolated version.
         """
         # When a Hosts object exists in this service, use that one, and ignore
         # the hosts which are passed in this constructor. So, any host
         # mappings do not apply
         # Note that when is_isolated=True, we always accept the hosts parameter, we asume that
         # the non-isolated base defines the correct Hosts
-        if hasattr(self.__class__, 'Hosts') and not is_isolated:
+        if hasattr(self.__class__, 'Hosts') and not made_array_cell:
             def get_hosts_container():
                 hosts = { }
                 for k in dir(self.__class__.Hosts):
