@@ -4,9 +4,8 @@ import termcolor
 import traceback
 
 from deployer.cli import CLInterface, Handler, HandlerType
-from deployer.exceptions import ExecCommandFailed, QueryException
 from deployer.service import ActionException
-from deployer.console import lesspipe, in_columns
+from deployer.console import Console
 from itertools import groupby
 import deployer
 
@@ -163,7 +162,10 @@ def Connect(self):
     from deployer.contrib.services import connect
     from deployer.host_container import HostsContainer
     c = connect.Connect(HostsContainer({ 'host': self.service.hosts._all }))
-    c.with_host().run(self.shell.pty, self.shell.logger_interface)
+
+    # Run as any other action. (Nice exception handling, e.g. in case of NoInput on host selection.)
+    Action(c.with_host, self.shell, False)()
+
 
 @create_navigable_handler
 def Find(self):
@@ -185,6 +187,8 @@ def Inspect(self):
     """
     Inspection of the current service. Show host mappings and other information.
     """
+    console = Console(self.shell.pty)
+
     def inspect():
         # Print full name
         yield termcolor.colored('  Service:    ', 'cyan') + \
@@ -240,7 +244,7 @@ def Inspect(self):
             for name, a in self.service.get_actions():
                 yield termcolor.colored(name, 'red'), len(name)
 
-        for line in in_columns(item_iterator(), self.shell.pty, margin_left=13):
+        for line in console.in_columns(item_iterator(), margin_left=13):
             yield line
 
         # Services
@@ -263,10 +267,10 @@ def Inspect(self):
                     yield text, length
 
             # Show in columns
-            for line in in_columns(item_iterator(), self.shell.pty, margin_left=13):
+            for line in console.in_columns(item_iterator(), margin_left=13):
                 yield line
 
-    lesspipe(inspect(), self.shell.pty)
+    console.lesspipe(inspect())
 
 
 @create_navigable_handler
@@ -280,6 +284,7 @@ def Ls(self):
     List subservices and actions in the current service.
     """
     w = self.shell.stdout.write
+    console = Console(self.shell.pty)
 
     def run():
         # Print services
@@ -288,7 +293,7 @@ def Ls(self):
             def column_iterator():
                 for name, service in self.service.get_subservices():
                     yield termcolor.colored(name, type_of_service(service).color), len(name)
-            for line in in_columns(column_iterator(), self.shell.pty):
+            for line in console.in_columns(column_iterator()):
                 yield line
 
         # Print actions
@@ -297,10 +302,10 @@ def Ls(self):
             def column_iterator():
                 for name, action in self.service.get_actions():
                     yield termcolor.colored(name, type_of_action(action).color), len(name)
-            for line in in_columns(column_iterator(), self.shell.pty):
+            for line in console.in_columns(column_iterator()):
                 yield line
 
-    lesspipe(run(), self.shell.pty)
+    console.lesspipe(run())
 
 @create_navigable_handler
 def SourceCode(self):
@@ -492,11 +497,11 @@ class Action(Handler):
 
         try:
             if sandbox:
-                result = self.action(*self.args, _trace_action=True).sandbox(pty, logger_interface)
+                result = self.action(*self.args).sandbox(pty, logger_interface)
             else:
-                result = self.action(*self.args, _trace_action=True).run(pty, logger_interface)
+                result = self.action(*self.args).run(pty, logger_interface)
 
-            action_callback.set_succeeded(result)
+            action_callback.set_succeeded()
 
             # When the result is a subservice, start a subshell.
             def handle_result(result):
@@ -511,7 +516,9 @@ class Action(Handler):
 
             if isinstance(result, list):
                 for r in result:
-                    handle_result(r.result)
+                    handle_result(r)
+            else:
+                handle_result(result)
 
         except ActionException, e:
             action_callback.set_failed(e)
@@ -519,8 +526,9 @@ class Action(Handler):
         except Exception, e:
             # Print traceback and return to shell
             print repr(e)
-            print traceback.format_exc()
-            action_callback.set_failed(e, traceback.format_exc())
+            tb = traceback.format_exc()
+            action_callback.set_failed(e, traceback=tb)
+
 
     def complete_subhandlers(self, part):
         # Autocompletion for first action parameter
@@ -627,6 +635,11 @@ class ShellState(object):
         self._service = subservice
         self._prev_service = None
 
+    def clone(self):
+        s = ShellState(self._service, self._return_state)
+        s._prev_service = self._prev_service
+        return s
+
     @property
     def prompt(self):
         # Returns a list of (text,color) tuples for the prompt.
@@ -669,7 +682,7 @@ class Shell(CLInterface):
         self.logger_interface = logger_interface
 
         if clone_shell:
-            self.state = clone_shell.state
+            self.state = clone_shell.state.clone()
         else:
             self._reset_navigation()
 
